/* `timescale 1ns / 1ps
*/
module audio(
SYSTEM_CLOCK,

ac97_bit_clock,	// Generated by AC'97
ac97_sdata_in,		// Serial data from AC'97

ac97_sdata_out,	// Serial data to AC'97
ac97_synch,			// Defines boundries of AC'97 frames, controls warm reset
audio_reset_b,		// AC'97 codec cold reset
Lcontrol,
Rcontrol,
dcm_locked,
LVibAct, 
RVibAct,
LOctave,
ROctave
);

input SYSTEM_CLOCK, ac97_bit_clock, ac97_sdata_in, LVibAct, RVibAct, LOctave, ROctave;
input [31:0] Lcontrol;
input [31:0] Rcontrol;
input dcm_locked;
output ac97_sdata_out, ac97_synch, audio_reset_b;

wire [19:0] left_in_data, right_in_data, left_out_data, right_out_data;
wire [3:0] Lnotes, Rnotes;
wire Lactivate, Ractivate, LVibAct, RVibAct, LOctave, ROctave;
wire [15:0] Lvolume, Rvolume;
wire dcm_locked;
wire reset;

/*	The Lcontrol register is broken down into volume, note, and activation controls
 *	The same goes for the Rcontrol register
 */
assign Lvolume = Lcontrol[23:4];
assign Lnotes = Lcontrol[3:0];
assign Lactivate = Lcontrol[31];
assign Rvolume = Rcontrol[23:4];
assign Rnotes = Rcontrol[3:0];
assign Ractivate = Rcontrol[31];

assign reset = !dcm_locked;

/*	Audio module
 *	This, and it's sub-modules, were created by Embedded Computing
 */
audio_sys audio0 (SYSTEM_CLOCK, reset, left_in_data, left_out_data, right_in_data, right_out_data, ready,
	      audio_reset_b, ac97_sdata_out, ac97_sdata_in,
         ac97_synch, ac97_bit_clock	  
			);

/*	Processing module: Takes in L and R control signals (volume, note, activate, vibrato, octave), 
 *	generates the tones, modifies them as needed, and outputs waves to the left_out and right_out
 *	lines to be used by the audio_sys module above				  
 */
processing processing0 (SYSTEM_CLOCK, ready, left_in_data, right_in_data, left_out_data, right_out_data, Lnotes, Rnotes, Lactivate, Ractivate, Lvolume, Rvolume, LVibAct, RVibAct, LOctave, ROctave);

endmodule
