`timescale 1ns / 1ps

module processing(clock, ready, left_in, right_in, left_out, right_out, Lnotes, Rnotes, Lactivate, Ractivate, Lvolume, Rvolume, LVibAct, RVibAct, LOctave, ROctave);
   input clock;              // system clock

   input ready, Lactivate, Ractivate, LVibAct, RVibAct, LOctave, ROctave;
   input [19:0] left_in, right_in;							// PCM data in from AC'97
   input [3:0] Lnotes, Rnotes;
   input [19:0] Lvolume, Rvolume;
   output [19:0] left_out, right_out;	// PCM data out to AC'97

	reg [19:0] left_out;
	reg [19:0] right_out;
	
	wire [25:0] Lfrequency, Rfrequency, LfrequencyVibrato, RfrequencyVibrato, LfrequencyOct, RfrequencyOct;
	wire LVibAct, RVibAct;
	wire Lactivate, Ractivate, LOctave, ROctave;
	wire [19:0] Lvolume, Rvolume;
	wire [19:0] Lwave, Rwave;

	//Decode the desired note for each channel into a 'frequency' value
	FrequencyDecoder LFrequencyDecoder(Lnotes, Lfrequency);
	FrequencyDecoder RFrequencyDecoder(Rnotes, Rfrequency);
	
	//Octave shift these frequencies if desired
	octaveShift LOct(clock, Lfrequency, LfrequencyOct, LOctave);
	octaveShift ROct(clock, Rfrequency, RfrequencyOct, ROctave);
	
	//Add vibrato if desired
	vibratoGenerator LVibGen(clock, LfrequencyOct, LfrequencyVibrato, LVibAct);
	vibratoGenerator RVibGen(clock, RfrequencyOct, RfrequencyVibrato, RVibAct);
	
	//Generate a PCM sine wave based on the modified frequency value
	pcm_generator pcmgenLeft(clock, LfrequencyVibrato, Lwave);
	pcm_generator pcmgenRight(clock, RfrequencyVibrato, Rwave);
	
	//Shift the sine wave up so it is entirely positive, and pass it
	//to the left and right outputs, which will cause the tones
	//to be generated on the LINE OUT of the FPGA board
	always@(posedge clock)
	begin
		if(Lactivate)
			left_out <= (Lwave + 524287) >> Lvolume;
		if(Ractivate)
			right_out <= (Rwave + 524287) >> Rvolume;
	end

endmodule
