/*###############################################
##      
##      Paddle Detector for XUPV2P
##
##      Verilog Source File (.v)
##
##      Original Created By: Jeffrey Goeders
##		Edited by James Durst, Stuart Byma, CyuYeol (Brian) Rhee
##			EDITED CODE WILL BE INDICATED BY COMMENTS THAT BEGIN: "EDITED CODE: "
##
##      Original Date: February 24, 2010
##
###############################################*/


module paddle_detector
(
	// System
    i_sys_rst,
    i_clk_100_bus,
	
	// Bus Master
	o_M_abort,
    o_M_ABus,
    o_M_BE,
    o_M_busLock,
    o_M_TAttribute,
    o_M_lockErr,
    o_M_mSize,
    o_M_priority,
    o_M_rdBurst,
    o_M_request,
    o_M_RNW,
    o_M_size,
    o_M_type,
    o_M_wrBurst,
    o_M_wrDBus,
    i_PLB_MAddrAck,
    i_PLB_MBusy,
    i_PLB_MRdErr,
    i_PLB_MWrErr,
    i_PLB_MRdBTerm,
    i_PLB_MRdDAck,
    i_PLB_MRdDBus,
    i_PLB_MRdWdAddr,
    i_PLB_MRearbitrate,
    i_PLB_MSSize,
    i_PLB_MWrBTerm,
    i_PLB_MWrDAck,
    i_PLB_MTimeout,
    i_PLB_PAValid,
    i_Sl_addrAck,
    i_Sl_wrDAck,
	i_Sl_rdDAck,
    i_Sl_wrComp,
	i_Sl_rdDBus,
    
	o_Paddle1_X1,
	o_Paddle1_Y1,
	o_Paddle1_X2,
	o_Paddle1_Y2,
	o_Paddle2_X1,
	o_Paddle2_Y1,
	o_Paddle2_X2,
	o_Paddle2_Y2,
	
    //MPMC
    i_MPMC_Done_Init,
    
    //Debug
    o_DBG_fsm_cs,
    o_DBG_general_purpose
);
	////////////////////////////////////////////////////////////
	//////////////////////////  PRE-COMPILER FUNCTIONS /////////
	////////////////////////////////////////////////////////////

    function integer CLogB2;
        input [31:0] Depth;
        integer i;

        begin
            i = Depth;      
            for(CLogB2 = 0; i > 0; CLogB2 = CLogB2 + 1)
            begin
                i = i >> 1;
            end
        end
    endfunction
	
	
	////////////////////////////////////////////////////////////
	////////////////////////// PARAMETERS //////////////////////
	////////////////////////////////////////////////////////////

	// Assigned by Xilinx Tools
	parameter   					C_PLBV46_NUM_SLAVES 	= 1;
	parameter   					C_PLBV46_DWIDTH     	= 32;
	
	// Inherited From MPD file
	parameter   					C_VIDEO_RAM_BASEADDR			= 32'h00000000;
	parameter   					C_PADDLE_COLOR_RAGE_BASEADDR	= 32'h00000000;
	parameter   					C_PADDLE_LOCATION_BASEADDR		= 32'h00000000;
	parameter						C_BYTES_PER_LINE				= 4096;

	// Local
	localparam  					C_BE_BITS           	= (C_PLBV46_DWIDTH / 8);
	
	localparam						C_BURST_LENGTH			= 16;	// Always write 16 times per request (maximum per burst)
	localparam						C_BYTES_PER_BURST		= C_BURST_LENGTH * 4;
	localparam						C_LINES_PER_FRAME		= 480;
	localparam						C_LINE_CNT_BITS			= CLogB2(C_LINES_PER_FRAME);
	localparam						C_PIXELS_PER_LINE		= 640;
	localparam						C_PIXELS_PER_LINE_BITS	= CLogB2(C_PIXELS_PER_LINE);
	localparam						C_BURSTS_PER_LINE 		= C_PIXELS_PER_LINE / C_BURST_LENGTH;
	localparam						C_BURST_CNT_BITS		= CLogB2(C_BURSTS_PER_LINE);
	localparam						C_REGION_ROWS			= 3;
	localparam						C_REGION_COLS			= 4;
	localparam						C_REGIONS 				= C_REGION_ROWS * C_REGION_COLS;

    //FSM States
	localparam						C_STATE_BITS 			= 4;
	localparam	[C_STATE_BITS-1:0]	S_INIT 			        = 0,
									S_READ_COL_RANGE_REQUEST= 1,
									S_READ_COL_RANGE		= 2,
									S_NEW_FRAME				= 3,
									S_NEW_LINE				= 4,
									S_READ_REQUEST			= 5,
									S_READ					= 6,
									S_READ_DONE				= 7,	
									S_CALCULATE				= 8,
									S_WRITE_REQUEST			= 9,
									S_WRITE					= 10,
									S_WRITE_DONE			= 11;
                                    

	////////////////////////////////////////////////////////////
	////////////////////////// PORTS ///////////////////////////
	////////////////////////////////////////////////////////////

	// System
	input                           		i_sys_rst;
	input                           		i_clk_100_bus;

	// PLB Master
	output                                  o_M_abort;
	output  [0:31]                          o_M_ABus;
	output  [0:(C_BE_BITS-1)]               o_M_BE;
	output                                  o_M_busLock;
	output  [0:15]                          o_M_TAttribute;
	output                                  o_M_lockErr;
	output  [0:1]                           o_M_mSize;
	output  [0:1]                           o_M_priority;
	output                                  o_M_rdBurst;
	output                                  o_M_request;
	output                                  o_M_RNW;
	output  [0:3]                           o_M_size;
	output  [0:2]                           o_M_type;
	output                                  o_M_wrBurst;
	output  [0:(C_PLBV46_DWIDTH-1)]         o_M_wrDBus;
	input                                   i_PLB_MAddrAck;
	input                                   i_PLB_MBusy;
	input                                   i_PLB_MRdErr;
	input                                   i_PLB_MWrErr;
	input                                   i_PLB_MRdBTerm;
	input                                   i_PLB_MRdDAck;
	input   [0:(C_PLBV46_DWIDTH-1)]         i_PLB_MRdDBus;
	input   [0:3]                           i_PLB_MRdWdAddr;
	input                                   i_PLB_MRearbitrate;
	input   [0:1]                           i_PLB_MSSize;
	input                                   i_PLB_MWrBTerm;
	input                                   i_PLB_MWrDAck;
	input                                   i_PLB_MTimeout;
	input                                   i_PLB_PAValid;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_addrAck;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_wrDAck;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_rdDAck;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_wrComp;
	input	[0:(C_PLBV46_DWIDTH-1)]			i_Sl_rdDBus;

	output	[9:0]							o_Paddle1_X1;
	output	[9:0]							o_Paddle1_Y1;
	output	[9:0]							o_Paddle1_X2;
	output	[9:0]							o_Paddle1_Y2;
	output	[9:0]							o_Paddle2_X1;
	output	[9:0]							o_Paddle2_Y1;
	output	[9:0]							o_Paddle2_X2;
	output	[9:0]							o_Paddle2_Y2;
	
	
    //MPMC
    input                                   i_MPMC_Done_Init;
    
    //Debug
    output  [3:0]                           o_DBG_fsm_cs;
    output  [3:0]                           o_DBG_general_purpose;
    
	////////////////////////////////////////////////////////////
	////////////////////////// DECLARATIONS ////////////////////
	////////////////////////////////////////////////////////////

 	// State Machine
	reg		[C_STATE_BITS-1:0]				r_cs;
	reg		[C_STATE_BITS-1:0]				c_ns;

    // PLB Master Write
	reg		[0:31]							r_M_ABus;
	reg		[C_BURST_CNT_BITS-1:0]			r_burst_cnt;
	wire									w_M_request;
	reg										c_M_wrBurst;
	reg										c_M_rdBurst;
    reg     [0:(C_PLBV46_DWIDTH-1)]			c_M_wrDBus;
	reg		[0:(C_BE_BITS-1)]				r_M_BE;
	reg		[3:0]							r_read_write_cnt;
	wire									w_M_RNW;
	reg		[C_LINE_CNT_BITS-1:0]			r_line_cnt;

    //Debug
    reg     [3:0]                   		c_DBG_fsm_cs;
    wire    [3:0]                   		w_DBG_general_purpose;
	reg		[7:0]							r_DBG_wait_cnt;
	
	// Paddle Location
	reg		[C_PIXELS_PER_LINE_BITS-1:0]	r_paddle_location_0_x;
	reg		[C_LINE_CNT_BITS-1:0]			r_paddle_location_0_y;
	reg		[C_PIXELS_PER_LINE_BITS-1:0]	r_paddle_location_1_x;
	reg		[C_LINE_CNT_BITS-1:0]			r_paddle_location_1_y;
	reg		[C_PIXELS_PER_LINE_BITS-1:0]	r_paddle_location_2_x;
	reg		[C_LINE_CNT_BITS-1:0]			r_paddle_location_2_y;
	reg		[C_PIXELS_PER_LINE_BITS-1:0]	r_paddle_location_3_x;
	reg		[C_LINE_CNT_BITS-1:0]			r_paddle_location_3_y;
	
	// Color Ranges
	reg		[5:0]							r_color_range_0_blue_low;
	reg		[5:0]							r_color_range_0_blue_high;
	reg		[5:0]							r_color_range_0_red_low;
	reg		[5:0]							r_color_range_0_red_high;
	reg		[5:0]							r_color_range_0_green_low;
	reg		[5:0]							r_color_range_0_green_high;
	reg		[5:0]							r_color_range_1_blue_low;
	reg		[5:0]							r_color_range_1_blue_high;
	reg		[5:0]							r_color_range_1_red_low;
	reg		[5:0]							r_color_range_1_red_high;
	reg		[5:0]							r_color_range_1_green_low;
	reg		[5:0]							r_color_range_1_green_high;
	
	//
	wire 		[5:0]						w_red_from_slave;
	wire		[5:0]						w_green_from_slave;
	wire		[5:0]						w_blue_from_slave;
	
	//EDITED CODE: The following were added
	parameter threshold = 16'd5;
	reg 	[15:0]	counter;
	reg		[15:0]	counter2;
	reg 			first;
	reg				first2;
	
	////////////////////////////////////////////////////////////
	////////////////////////// ASSIGNMENTS /////////////////////
	////////////////////////////////////////////////////////////
    
    // Output to PLB Bus
	assign 	o_M_abort				= 1'b0;						// Not Used
	assign	o_M_ABus				= r_M_ABus;
	assign	o_M_BE					= r_M_BE;
	assign	o_M_busLock				= 1'b0;						// Not Used
	assign	o_M_TAttribute			= 16'b0;					// Not Used
	assign 	o_M_lockErr				= 1'b0;						// Not Used
	assign 	o_M_mSize				= 2'b0;						// 32-bit bus size
	assign 	o_M_priority			= 2'b11;					// Always highest priority
	assign 	o_M_rdBurst				= c_M_rdBurst;
	assign 	o_M_request				= w_M_request;	
	assign 	o_M_RNW					= w_M_RNW;
	assign 	o_M_size				= 4'hA;						// Burst of Word Size (32-bit) Transfers
	assign 	o_M_type				= 3'b000;					// Standard Memory Transfer
	assign	o_M_wrBurst				= c_M_wrBurst;		
	assign 	o_M_wrDBus				= c_M_wrDBus;
	
    // PLB Signal Logic
    assign	w_M_request				= ( r_cs == S_WRITE_REQUEST || r_cs == S_READ_REQUEST || 
										r_cs == S_READ_COL_RANGE_REQUEST );
	assign	w_M_RNW					= ( r_cs == S_READ_REQUEST || r_cs == S_READ_COL_RANGE_REQUEST);
	
	assign w_red_from_slave = i_Sl_rdDBus[8:13];
	assign w_green_from_slave = i_Sl_rdDBus[16:21];
	assign w_blue_from_slave = i_Sl_rdDBus[24:29];
	
    //Debug 
    assign  w_DBG_general_purpose   = w_red_from_slave[4:1];
    assign  o_DBG_fsm_cs            = c_DBG_fsm_cs;
    assign  o_DBG_general_purpose   = w_DBG_general_purpose;
	
	assign o_Paddle1_X1 = r_paddle_location_0_x;
	assign o_Paddle1_X2 = r_paddle_location_1_x;
	assign o_Paddle1_Y1 = r_paddle_location_0_y;
	assign o_Paddle1_Y2 = r_paddle_location_1_y;
	
	assign o_Paddle2_X1 = r_paddle_location_2_x;
	assign o_Paddle2_X2 = r_paddle_location_3_x;
	assign o_Paddle2_Y1 = r_paddle_location_2_y;
	assign o_Paddle2_Y2 = r_paddle_location_3_y;

	////////////////////////////////////////////////////////////
	///////////////////// COMBINATIONAL LOGIC //////////////////
	////////////////////////////////////////////////////////////

	// c_ns
	always @ ( *)
	begin
		case(r_cs)
		S_INIT:
		begin
			if ( (i_MPMC_Done_Init == 1'b1) && (r_DBG_wait_cnt >= 150) )
			begin
				c_ns <= S_READ_COL_RANGE_REQUEST;
			end
			else
			begin
				c_ns <= S_INIT;
			end
		end
		S_READ_COL_RANGE_REQUEST:
		begin
			if ( i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1 )
			begin
				c_ns <= S_READ_COL_RANGE;
			end
			else
			begin
				c_ns <= S_READ_COL_RANGE_REQUEST;
			end
		end
		S_READ_COL_RANGE:
		begin
			if ( r_read_write_cnt == 4'd11 )
			begin
				c_ns <= S_NEW_FRAME;
			end
			else
			begin
				c_ns <= S_READ_COL_RANGE;
			end
		end
		S_NEW_FRAME:
		begin
            c_ns <= S_NEW_LINE;
		end
		S_NEW_LINE:
		begin
			c_ns <= S_READ_REQUEST;
		end
		S_READ_REQUEST:
		begin
			if ( i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1 )
			begin
				c_ns <= S_READ;
			end
			else
			begin
				c_ns <= S_READ_REQUEST;
			end
		end
		S_READ:
		begin
			if ( r_read_write_cnt == 4'd14 )
			begin
				c_ns <= S_READ_DONE;
			end
			else
			begin
				c_ns <= S_READ;
			end
		end
		S_READ_DONE:
		begin			
			if (r_burst_cnt == C_BURSTS_PER_LINE) // Line Done
			begin
				if (r_line_cnt == C_LINES_PER_FRAME)
				begin
					c_ns <= S_CALCULATE;
				end
				else
				begin
					c_ns <= S_NEW_LINE;
				end
			end
			else // Line Incomplete
			begin
				c_ns <= S_READ_REQUEST;
			end
		end
		S_CALCULATE:
		begin
			c_ns <= S_WRITE_REQUEST;
		end
		S_WRITE_REQUEST:
		begin
			if ( i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1 )
			begin
				c_ns <= S_WRITE;
			end
			else
			begin
				c_ns <= S_WRITE_REQUEST;
			end
		end
		S_WRITE:
		begin
			if ( i_Sl_wrComp == 1'b1 )
			begin
				c_ns <= S_WRITE_DONE;
			end
			else
			begin
				c_ns <= S_WRITE;
			end
		end
		S_WRITE_DONE:
		begin			
			c_ns <= S_READ_COL_RANGE_REQUEST;
		end
		default:
		begin
			c_ns <= S_INIT;
		end
		endcase
	end
	
	//c_M_wrBurst
	always @ (*)
	begin
		if ( r_cs == S_WRITE_REQUEST )
		begin
			c_M_wrBurst <= 1'b1;
		end
		else if ( (r_cs == S_WRITE)	&& (i_Sl_wrComp == 1'b0) )
		begin
			c_M_wrBurst <= 1'b1;
		end
		else
		begin
			c_M_wrBurst <= 1'b0;
		end
	end

	//c_M_rdBurst
	always @ (*)
	begin
		if ( r_cs == S_READ_REQUEST || r_cs == S_READ_COL_RANGE_REQUEST)
		begin
			c_M_rdBurst <= 1'b1;
		end
		else if ( (r_cs == S_READ)	&& (r_read_write_cnt <= 4'd14) )
		begin
			c_M_rdBurst <= 1'b1;
		end
		else if ( (r_cs == S_READ_COL_RANGE) && (r_read_write_cnt <= 11) )
		begin
			c_M_rdBurst <= 1'b1;
		end
		else
		begin
			c_M_rdBurst <= 1'b0;
		end
	end
	
    //c_DBG_fsm_cs
    always @ (*)
    begin
        case (r_cs)
        S_INIT:
        begin
            c_DBG_fsm_cs <= 0;
        end        
		S_READ_COL_RANGE_REQUEST:
		begin
			c_DBG_fsm_cs <= 1;
		end
		S_READ_COL_RANGE:
		begin
			c_DBG_fsm_cs <= 2;
		end
        S_NEW_FRAME:
        begin
            c_DBG_fsm_cs <= 3;  
        end
        S_NEW_LINE:
        begin
            c_DBG_fsm_cs <= 4;
        end
        S_READ_REQUEST:
        begin
            c_DBG_fsm_cs <= 5;
        end
        S_READ:
        begin
            c_DBG_fsm_cs <= 6;
        end
		S_READ_DONE:
        begin
            c_DBG_fsm_cs <= 7;
        end        
        S_CALCULATE:
        begin
            c_DBG_fsm_cs <= 8;  
        end
        S_WRITE_REQUEST:
        begin
            c_DBG_fsm_cs <= 9;
        end
        S_WRITE:
        begin
            c_DBG_fsm_cs <= 10;
        end
        S_WRITE_DONE:
        begin
            c_DBG_fsm_cs <= 11;
        end
        default
        begin
            c_DBG_fsm_cs <= {C_STATE_BITS{1'b1}};
        end
        endcase
    end
    
    //c_M_wrDBus
    always @ (*)
    begin
		case (r_read_write_cnt)
		4'd0:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_0_x};
		end
		4'd1:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_0_y};
		end
		4'd2:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_1_x};
		end
		4'd3:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_1_y};
		end
		4'd4:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_2_x};
		end
		4'd5:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_2_y};
		end
		4'd6:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_3_x};
		end
		4'd7:
		begin
			c_M_wrDBus <= {24'b0, r_paddle_location_3_y};
		end
		default:
		begin
			c_M_wrDBus <= 0;
		end
        endcase
    end
  
    
	////////////////////////////////////////////////////////////
	///////////////////// CLOCKED ASSIGNMENTS //////////////////
	////////////////////////////////////////////////////////////

	// r_cs
	always @ (posedge i_sys_rst or posedge i_clk_100_bus)
	begin
		if ( i_sys_rst == 1'b1 )
		begin
			r_cs <= S_INIT;
		end
		else
		begin
			r_cs <= c_ns;
		end
	end
	
	// r_M_BE
	always @ ( posedge i_clk_100_bus )
	begin
		if (( r_cs == S_INIT ) || ( r_cs == S_WRITE_DONE )) // About to do color range read
		begin
			r_M_BE <= 11; //Read 12 words
		end
		if ( r_cs == S_NEW_FRAME ) //About to begin reads
		begin
			r_M_BE <= C_BURST_LENGTH - 1;		// Burst Length (16)
		end
		else if ( r_cs == S_CALCULATE ) 
		begin
			r_M_BE <= 7; // Write 8 words (paddle x,y x4)
		end
	end
	
	// r_read_write_cnt
	always @ ( posedge i_clk_100_bus )
	begin
		if ( (r_cs == S_READ_REQUEST) || ( r_cs == S_WRITE_REQUEST) ||
			(r_cs == S_READ_COL_RANGE_REQUEST) )
		begin
			r_read_write_cnt <= 4'b0;
		end
		else if ( ((r_cs == S_READ) || (r_cs == S_READ_COL_RANGE)) 
				&& (i_Sl_rdDAck == 1'b1) )
		begin
			r_read_write_cnt <= r_read_write_cnt + 4'b0001;
		end
		else if ( (r_cs == S_WRITE) && (i_Sl_wrDAck == 1'b1) )
		begin
			r_read_write_cnt <= r_read_write_cnt + 4'b0001;
		end
	end

	//r_burst_cnt
	always @ (posedge i_clk_100_bus)
	begin
		if (r_cs == S_NEW_LINE)
		begin
			r_burst_cnt <= {C_BURST_CNT_BITS{1'b0}};
		end
		else if ( (r_cs == S_READ_REQUEST) && (c_ns == S_READ) )
		begin
			r_burst_cnt <= r_burst_cnt + 1;
		end			
	end
	
	//r_M_ABus
	always @ (posedge i_clk_100_bus)
	begin
		if ( r_cs == S_NEW_LINE  )
		begin
			r_M_ABus <= C_VIDEO_RAM_BASEADDR + (C_BYTES_PER_LINE * r_line_cnt);
		end
		else if ( r_cs == S_READ_DONE )
		begin
			r_M_ABus <= r_M_ABus + C_BYTES_PER_BURST;
		end
		else if ( r_cs == S_CALCULATE )
		begin
			r_M_ABus <= C_PADDLE_LOCATION_BASEADDR;
		end
		else if ( r_cs == S_INIT || r_cs == S_WRITE_DONE ) //About to read paddle col range
		begin
			r_M_ABus <= C_PADDLE_COLOR_RAGE_BASEADDR;
		end
	end
    
     //r_line_cnt
     always @ (posedge i_clk_100_bus)
     begin
        if ( r_cs == S_NEW_FRAME)
        begin
			r_line_cnt <= {C_LINE_CNT_BITS{1'b0}};
        end
        else if ( r_cs == S_NEW_LINE )
        begin
            r_line_cnt <= r_line_cnt + 1;
        end
     end
	
	always @ (posedge i_clk_100_bus)
	begin
		if (i_sys_rst == 1'b1)
		begin
			r_DBG_wait_cnt <= 0;
		end
		else if ((r_cs == S_INIT) && (i_MPMC_Done_Init == 1'b1))
		begin
			r_DBG_wait_cnt <= r_DBG_wait_cnt + 1;
		end
	end
	

	
	//r_color_range_0_red_low;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 0 )
		begin
			r_color_range_0_red_low <= i_Sl_rdDBus[26:31];
		end 
	end
	
	//r_color_range_0_red_high;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 1 )
		begin
			r_color_range_0_red_high <= i_Sl_rdDBus[26:31];
		end
	end
	
	//r_color_range_0_green_low;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 2 )
		begin
			r_color_range_0_green_low <= i_Sl_rdDBus[26:31];
		end 
	end
	
	//r_color_range_0_green_high;
	always @ ( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 3 )
		begin
			r_color_range_0_green_high <= i_Sl_rdDBus[26:31];
		end
	end
	
		//r_color_range_0_blue_low;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 4 )
		begin
			r_color_range_0_blue_low <= i_Sl_rdDBus[26:31];
		end 
	end
	
	//r_color_range_0_blue_high;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 5 )
		begin
			r_color_range_0_blue_high <= i_Sl_rdDBus[26:31];
		end 
	end
	
	//r_color_range_1_red_low;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 6 )
		begin
			r_color_range_1_red_low <= i_Sl_rdDBus[26:31];
		end 
	end
	
	//r_color_range_1_red_high;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 7 )
		begin
			r_color_range_1_red_high <= i_Sl_rdDBus[26:31];
		end
	end
	
	//r_color_range_1_green_low;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 8 )
		begin
			r_color_range_1_green_low <= i_Sl_rdDBus[26:31];
		end 
	end
	
	//r_color_range_1_green_high;
	always @ ( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 9 )
		begin
			r_color_range_1_green_high <= i_Sl_rdDBus[26:31];
		end
	end
	
		//r_color_range_1_blue_low;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 10 )
		begin
			r_color_range_1_blue_low <= i_Sl_rdDBus[26:31];
		end 
	end
	
	//r_color_range_1_blue_high;
	always @( posedge i_clk_100_bus )
	begin
		if ( r_cs == S_READ_COL_RANGE && i_Sl_rdDAck == 1'b1 && r_read_write_cnt == 11 )
		begin
			r_color_range_1_blue_high <= i_Sl_rdDBus[26:31];
		end 
	end
	
	// r_paddle_location_x;
	// r_paddle_location_y;
	always @ ( posedge i_clk_100_bus )
		begin		
		
		//EDITED CODE: This if statement was added in order to reset signals
		//We want to detect the first and last pixel of a specfic color, so 
		//we must have a way of making certain that we set a different point when
		//we find the first pixel - the first and first2 values do this
		//The counters count the number of consecutive points of the same
		//color, and so should be reset every frame
		if(r_cs == S_NEW_FRAME)
			begin
				first <= 1;
				first2 <= 1;
				counter <= 0;
				counter2 <= 0;
			end
		
		if  ( 	r_cs == S_READ && i_Sl_rdDAck == 1'b1 && 
				w_red_from_slave >= r_color_range_0_red_low &&
				w_red_from_slave <= r_color_range_0_red_high &&
				w_green_from_slave >= r_color_range_0_green_low &&
				w_green_from_slave <= r_color_range_0_green_high &&
				w_blue_from_slave >= r_color_range_0_blue_low &&
				w_blue_from_slave <= r_color_range_0_blue_high 
			)
		begin
			//EDITED CODE:  The majority of the code beyong this point is our own
			//with the exception of the lines that assign values to r_paddle_location_
			
			//EDITED CODE: If we've seen enough consecutive pixels of the same color, then record the current location
			if(counter >= threshold)
			begin
				if ( first ) //if this is the first time this frame we've seen a valid color, write to one x,y pair
				begin
					r_paddle_location_0_x <= ( (r_burst_cnt - 1) * C_BURST_LENGTH ) + r_read_write_cnt;
					r_paddle_location_0_y <= r_line_cnt;
					first <= 0;
				end
				else		// otherwise, write to another
				begin
					r_paddle_location_1_x <= ( (r_burst_cnt - 1) * C_BURST_LENGTH ) + r_read_write_cnt;
					r_paddle_location_1_y <= r_line_cnt;
				end
			end
			else	//EDITED CODE: Pixel is the correct color, so increment the counter
			begin
				counter <= counter + 1;
			end
			counter2 <= 0;
		end
		else if (r_cs == S_READ && i_Sl_rdDAck == 1'b1 && 
				w_red_from_slave >= r_color_range_1_red_low &&
				w_red_from_slave <= r_color_range_1_red_high &&
				w_green_from_slave >= r_color_range_1_green_low &&
				w_green_from_slave <= r_color_range_1_green_high &&
				w_blue_from_slave >= r_color_range_1_blue_low &&
				w_blue_from_slave <= r_color_range_1_blue_high 
			)
		begin
			//EDITED CODE:  This code works the same as above, but is using a different set of color values
			//so a different color is looked for
			if(counter2 >= threshold)
			begin
				if (first2)
				begin
					r_paddle_location_2_x <= ( (r_burst_cnt - 1) * C_BURST_LENGTH ) + r_read_write_cnt;
					r_paddle_location_2_y <= r_line_cnt;
					first2 <= 0;
				end
				else
				begin
					r_paddle_location_3_x <= ( (r_burst_cnt - 1) * C_BURST_LENGTH ) + r_read_write_cnt;
					r_paddle_location_3_y <= r_line_cnt;
				end
			end
			else
			begin
				counter2 <= counter2 + 1;
			end
			counter <= 0;
		end
		//EDITED CODE: 	Neither color was found, so reset the counters for both
		else
		begin
			counter <= 0;
			counter2 <= 0;
		end
	end
	
	////////////////////////////////////////////////////////////
	////////////////////////// INSTANTIATIONS //////////////////
	////////////////////////////////////////////////////////////

endmodule
