#include <xbasic_types.h>
#include <xparameters.h>
#include <xiic_l.h>
#include <xgpio.h>
#include <math.h>

#define GPO_REG_OFFSET 0x124
#define DECODER_ADDR 0x20 //Read: 0x41, Write: 0x40
#define SEND_CNT 3
#define RECV_CNT 3
#define GPO_RESETS_OFF 1
#define GPO_RESET_IIC 3
#define GPO_RESET_DECODER 0
#define DISTANCE_THRESHOLD 45

//Required for the use of trigonometric functions
#define PI 3.14159265

//Definitions of musical notes that map to the values in the hardware frequency decoder
#define A 0
#define Bb 1
#define B 2
#define C 3
#define Db 4
#define D 5
#define Eb 6
#define E 7
#define F 8
#define Gb 9
#define G 10
#define A2b 11
#define A2 12
#define B2b 13
#define B2 14
#define C2 15

#define ON 0x80000000
#define ACT	0x1
#define DEACT 0x0

//The base addresses for the video frame, the color ranges, and the 'points
#define VIDEO_FRAME_BASEADDR 	0x40000000
#define COLOR_RANGE_BASEADDR 	0x42000000
#define POINTS_BASEADDR 		0x42001000

//define statements to alias GPIO channels
//for gpio (the audio controller)
#define CONTROL 	1
#define STATUS		2

//Defined values for use with the 'guitar' instrument
#define UP				0
#define DOWN 			1
#define STRUMLENGTH		10000

struct VideoModule {
  Xuint8 addr;
  Xuint8 config_val;
  Xuint8 actual_val;
};

//Composite Configuration constants
#define DECODER_COMP_CONFIG_CNT 18
struct VideoModule decoder_comp[] = { 
  { 0x00, 0x04, 0 },
  { 0x15, 0x00, 0 },
  { 0x17, 0x41, 0 },
  { 0x27, 0x58, 0 },
  { 0x3a, 0x16, 0 }, 
  { 0x50, 0x04, 0 },
  { 0x0e, 0x80, 0 },
  { 0x50, 0x20, 0 },
  { 0x52, 0x18, 0 }, 
  { 0x58, 0xed, 0 },
  { 0x77, 0xc5, 0 },
  { 0x7c, 0x93, 0 },
  { 0x7d, 0x00, 0 },
  { 0xd0, 0x48, 0 },
  { 0xd5, 0xa0, 0 },
  { 0xd7, 0xea, 0 },
  { 0xe4, 0x3e, 0 },
  { 0xea, 0x0f, 0 }, 
  { 0x0e, 0x00, 0 } };

void configDecoder(struct VideoModule *decoder, int config_cnt );
void write_pixel(unsigned int value, unsigned int x, unsigned int y);
void findLAngle(int x1, int y1, int x2, int y2);
void findRAngle(int x1, int y1, int x2, int y2);
void changeLVolume(unsigned int newVolume);
void changeRVolume(unsigned int newVolume);
void changeLNote(unsigned int newNote);
void changeRNote(unsigned int newNote);
void Lonoff(unsigned int onoff);
void Ronoff(unsigned int onoff);
int distanceCheck(int x1, int y1, int x2, int y2);
int distance(int x1, int y1, int x2, int y2);

//GPIO instance declations
XGpio Gpio;
XGpio points1x;
XGpio points1y;

//Values that allow for maintaining volume or notes across calss to the changeXVolume and changeXNote functions
unsigned int currentLVolume;
unsigned int currentLNote;
unsigned int currentRVolume;
unsigned int currentRNote;
unsigned int currentLActivate;
unsigned int currentRActivate;

int main() 
{
//The first thing done is initialization of the video decoder - code taken from the video to ram project posted on the class discussion board
	Xuint8 start_addr = 0;
	Xuint8 send_data[SEND_CNT] = {0};
	Xuint32 send_cnt;
	Xuint8 recv_data[RECV_CNT] = {0};
	Xuint32 recv_cnt;
	Xuint8 i;
	XStatus Status;
	recv_cnt = 0;
	
	print("\r\nBeginning Setup\r\n");
	configDecoder(decoder_comp, DECODER_COMP_CONFIG_CNT);
	print("\r\nSetup Complete!\r\n\r\n");

	//initialize the GPIOs
	//initialize Audio GPIO
    Status = XGpio_Initialize(&Gpio, XPAR_XPS_GPIO_0_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        print("GPIO initialization error on audio GPIO\n\r\r");
    }
	XGpio_SetDataDirection(&Gpio, CONTROL, 0x0);
	XGpio_SetDataDirection(&Gpio, STATUS, 0x0);	

	//initialize Paddle1x GPIO
    Status = XGpio_Initialize(&points1x, XPAR_XPS_GPIO_PADDLE1_X_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        print("GPIO initialization error on points1x GPIO\n\r\r");
    }
	XGpio_SetDataDirection(&points1x, 1, 0x1);
	XGpio_SetDataDirection(&points1x, 2, 0x1);	
	
	//initialize Paddle1x GPIO
    Status = XGpio_Initialize(&points1y, XPAR_XPS_GPIO_PADDLE1_Y_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        print("GPIO initialization error on points1y GPIO\n\r\r");
    }
	XGpio_SetDataDirection(&points1y, 1, 0x1);
	XGpio_SetDataDirection(&points1y, 2, 0x1);	

	// Configure the colours that will be detected for the points
	unsigned int * addr;
	addr = (unsigned int *) COLOR_RANGE_BASEADDR;
	// Red
	*(addr+0) = 0x20; //27
	*(addr+1) = 0xFF; //FF
	*(addr+2) = 0x00;  //00
	*(addr+3) = 0x12; //12
	*(addr+4) = 0x00;  //00
	*(addr+5) = 0x12; //12
	
	//Blue
	*(addr+6) = 0x00; //00
	*(addr+7) = 0x22; //12
	*(addr+8) = 0x4A;  //50
	*(addr+9) = 0x7A;  //70
	*(addr+10) = 0x30;  //20
	*(addr+11) = 0xFF; //FF
	
	//Turn the audio core on
	XGpio_DiscreteWrite(&Gpio, STATUS, ACT);
	long j;
	Xuint32 note;

	//These will be used to hold the coordinates of the points for the instruments (two points per instrument)
	unsigned volatile int Lx1, Lx2, Ly1, Ly2;
	unsigned volatile int Rx1, Rx2, Ry1, Ry2;

	unsigned volatile int * pointsptr = (unsigned int *)POINTS_BASEADDR;
	unsigned volatile int points;
	
	print("Here we go! Time to make some music!\r\n");
	Xuint32 volume = 0;
	
	//Turn on the volume for both instruments
	changeLVolume(0x00001); //max volume - volumes are given as attenuations
	changeRVolume(0x00001); //max volume - volumes are given as attenuations
	
	//for debugging - plays a looping sequence of notes 
	//to turn on, add a slash to the very end of the very next line
/**
	int loopy ;
	while(1)
	{
		currentLActivate = ON;
		currentRActivate = ON;
		for(loopy = 0; loopy < 10000000; loopy++){}
		changeLNote(A);
		changeRNote(B);
		for(loopy = 0; loopy < 10000000; loopy++){}
		changeLNote(B);
		changeRNote(C);
		for(loopy = 0; loopy < 10000000; loopy++){}
		changeLNote(C);
		changeRNote(D);
		for(loopy = 0; loopy < 10000000; loopy++){}
		changeLNote(D);
		changeRNote(E);
		for(loopy = 0; loopy < 10000000; loopy++){}
		changeLNote(E);
		changeRNote(F);
	}
/**/

	//Variables needed for the guitar instrument - strumPos keeps track of whether the last strum was up or down
	//guitarcount is a counter which is used to count down to turn the sound off after a set period of time (STRUMLENGTH)
	//strumX/Y and neckX/Y are used to keep track of which detected point is which
	int strumPos = UP, guitarcount;
	int strumX, strumY, neckX, neckY;
	int toggle_instrument = 0;

	//These will be used to detect key presses on the UART
	int * status_reg = (unsigned int *)(XPAR_UARTLITE_0_BASEADDR + 0x8);
	int * control_reg = (unsigned int *)(XPAR_UARTLITE_0_BASEADDR + 0xC);
	volatile int status = 0;
	*control_reg = 0x2;
	
	unsigned volatile int * testptr = (unsigned int *)VIDEO_FRAME_BASEADDR;
	unsigned volatile int test;
	unsigned int red, blue, green;
	
	while(1)
	{
		//After this, status is 1 if there is data on the UART incoming FIFO, and 0 otherwise
		status = *status_reg & 0x01; //load value from status register, and mask so we only look at bit 1 - tells if there is data on the FIFO
	
		//If a key was pressed on the FIFO, change the instrument which the second set of dots uses, and print the new instrument's name
		if(status)
		{
			toggle_instrument = 1 - toggle_instrument; //toggles between 0 and 1 (1 - 1 = 0, 1 - 0=1)
			*control_reg = 0x2; //binary 0010 - setting the second bit to 1 resets the receive register
			if(toggle_instrument)
				xil_printf("Trombone!\r");
			else
				xil_printf("Guitar!  \r");
		}
	 
		//Get the latest points detected by the hardware from memory to be used for playing sounds
		Lx1 = pointsptr[0];
		Ly1 = pointsptr[1];
		Lx2 = pointsptr[2];
		Ly2 = pointsptr[3];

		Rx1 = pointsptr[4];
		Ry1 = pointsptr[5];
		Rx2 = pointsptr[6];
		Ry2 = pointsptr[7];

		//Play a note based on the angle formed by the L points only if the points are sufficient far apart
		//see the 'distanceCheck()' function comments for further details
		if(distanceCheck(Lx1, Ly1, Lx2, Ly2))
		{
			currentLActivate = ON;
			findLAngle(Lx1, Ly1, Lx2, Ly2);
		}
		else
		{
			Lonoff(0);
		}

		if(toggle_instrument)
		{
			//As above, but here play a note based on the distance between the two points rather than the angle	
			if(distanceCheck(Rx1, Ry1, Rx2, Ry2))
			{
				currentRActivate = ON;
				distance(Rx1, Ry1, Rx2, Ry2);
			}
			else
			{
				Ronoff(0);
			}
		}
		else
		{
			/*Blue lights act as guitar*/
			//Make sure the right-most point is always considered as the 'neck' of the guitar
			//so that later we can track if the other point is above or below it for strumming
			if(Rx1 > Rx2)
			{
				strumX = Rx2;
				strumY = Ry2;
				neckX = Rx1;
				neckY = Ry1;
			}
			if(Rx1 < Rx2)
			{
				strumX = Rx1;
				strumY = Ry1;
				neckX = Rx2;
				neckY = Ry2;
			}

			//This will decrement until it reaches 0, and which point the guitar will turn off,
			//which means that the note will stop on it's own after a short time
			if(guitarcount)
			{
				guitarcount--;
			}//when it reaches 0, turn the guitar off
			else
			{
				Ronoff(0);
			}
			
			//Basically, if the 'strum' point has moved below the 'neck' point, then a strum from UP to DOWN has occured
			//set the note based on the distance of the strum from the neck, and set guitarcount to STRUMLENGTH so it turns
			//off after a little while
			if((strumPos == UP) && (strumY < neckY))
			{
					if(distanceCheck(Rx1, Ry1, Rx2, Ry2))
					{
						currentRActivate = ON;
						distance(Rx1, 0, Rx2, 0);
						guitarcount = STRUMLENGTH;
					}
					strumPos = DOWN;
			}
			//Basically, if the 'strum' point has moved above the 'neck' point, then a strum from DOWN to UP has occured
			//set the note based on the distance of the strum from the neck, and set guitarcount to STRUMLENGTH so it turns
			//off after a little while
			else if((strumPos == DOWN) && (strumY > neckY))
			{
					if(distanceCheck(Rx1, Ry1, Rx2, Ry2))
					{
						//play note
						currentRActivate = ON;
						distance(Rx1, 0, Rx2, 0);
						guitarcount = STRUMLENGTH;
					}
					strumPos = UP;
			}
		}
/**
		test = testptr[(240 * 1024) + 320];
		red = (test & 0x00FF0000) >> 16;
		blue = (test & 0x0000FF00) >> 8;
		green = (test & 0x000000FF);
		xil_printf("Middle pixel has RGB values: %d, %d, %d\r\n", red, blue, green);
		
		write_pixel(0x00FFFFFF, 319, 240);
/**/
/*Write blue crosshairs to screen to identify the locations detected as the L points*/
		write_pixel(0x000000FF, Lx1, Ly1);
		write_pixel(0x000000FF, Lx1+1, Ly1);
		write_pixel(0x000000FF, Lx1, Ly1+1);
		write_pixel(0x000000FF, Lx1-1, Ly1);
		write_pixel(0x000000FF, Lx1, Ly1-1);

		write_pixel(0x000000FF, Lx2, Ly2);
		write_pixel(0x000000FF, Lx2+1, Ly2);
		write_pixel(0x000000FF, Lx2, Ly2+1);
		write_pixel(0x000000FF, Lx2-1, Ly2);
		write_pixel(0x000000FF, Lx2, Ly2-1);

//Write yellow crosshairs to the screen to identify locations detected as the R points
		write_pixel(0x00EEEE00, Rx1, Ry1);
		write_pixel(0x00EEEE00, Rx1+1, Ry1);
		write_pixel(0x00EEEE00, Rx1, Ry1+1);
		write_pixel(0x00EEEE00, Rx1-1, Ry1);
		write_pixel(0x00EEEE00, Rx1, Ry1-1);

		write_pixel(0x00EEEE00, Rx2, Ry2);
		write_pixel(0x00EEEE00, Rx2+1, Ry2);
		write_pixel(0x00EEEE00, Rx2, Ry2+1);
		write_pixel(0x00EEEE00, Rx2-1, Ry2);
		write_pixel(0x00EEEE00, Rx2, Ry2-1);
	}

} 

//Writes into memory to overwrite the value of a pixel in the video frame
void write_pixel(unsigned int value, unsigned int x, unsigned int y)
{
	if( (x > 0) && (x < 640) && (y > 0) && (y <480) )
	( (unsigned int *) VIDEO_FRAME_BASEADDR)[(y * 1024) + x] = value;
}

//Configured the video decoder chip - this code was copied from the video to ram project posted on the course bulletin board
void configDecoder(struct VideoModule *decoder, int config_cnt ) 
{
  Xuint16 send_cnt, i;
  Xuint8 send_data[2] = {0};
  Xuint8 success = 1;
  send_cnt = 2;
  xil_printf("  Configuring Decoder...\t");
  //read in configuration constants and send them to the VDEC one by one
  for( i = 0; i < config_cnt; i++ )
   {

    send_data[0] = decoder[i].addr;

    send_data[1] = decoder[i].config_val;

    send_cnt = XIic_Send(XPAR_IIC_BASEADDR, DECODER_ADDR, send_data, 2,XIIC_STOP );

    if( send_cnt != 2 ) 
	 {
      xil_printf("Error writing to address %02x\r\n", decoder[i].addr);
      success = 0;
      break;
     }
   }

  if( success )
    xil_printf("SUCCESS!\r\n");

}

//This functions takes in two coordinates and plays a note based on the angle formed between them
//The right most point is always considered to be the 'origin'
//This note is output to the LEFT audio channel
void findLAngle(int x1, int y1, int x2, int y2)
{
	float angle = atan2(-(y1-y2), (x1-x2));
	int angle2 = (angle * 180)/PI;
	float slope = (y1-y2)/(x1-x2);

	if((angle2 > 70) && (angle2 < 88))
	{
		changeLNote(A);
	}
	else if((angle2 > 47) && (angle2 < 66))
	{
		changeLNote(B);
	}
	else if((angle2 > 25) && (angle2 < 43))
	{
		changeLNote(Db);
	}
	else if((angle2 > 2) && (angle2 < 20))
	{
		changeLNote(D);
	}
	else if((angle2 > 160) && (angle2 < 178))
	{
		changeLNote(E);
	}
	else if((angle2 > 137) && (angle2 < 156))
	{
		changeLNote(Gb);
	}
	else if((angle2 > 114) && (angle2 < 133))
	{
		changeLNote(A2b);
	}
	else if((angle2 > 92) && (angle2 < 110))
	{
		changeLNote(A2);
	}
}

//Just as above, but now output the note to the RIGHT audio channel
void findRAngle(int x1, int y1, int x2, int y2)
{
	float angle = atan2(-(y1-y2), (x1-x2));
	int angle2 = (angle * 180)/PI;
	float slope = (y1-y2)/(x1-x2);

	if((angle2 > 70) && (angle2 < 86))
	{
		changeRNote(A);
	}
	else if((angle2 > 47) && (angle2 < 64))
	{
		changeRNote(B);
	}
	else if((angle2 > 24) && (angle2 < 43))
	{
		changeRNote(C);
	}
	else if((angle2 > 2) && (angle2 < 20))
	{
		changeRNote(D);
	}
	else if((angle2 > 160) && (angle2 < 178))
	{
		changeRNote(E);
	}
	else if((angle2 > 137) && (angle2 < 156))
	{
		changeRNote(F);
	}
	else if((angle2 > 114) && (angle2 < 133))
	{
		changeRNote(G);
	}
	else if((angle2 > 92) && (angle2 < 110))
	{
		changeRNote(A2);
	}
}

//Change the volume of the LEFT audio channel
//Do NOT change the note or whether or not the channel is on
void changeLVolume(unsigned int newVolume)
{
	XGpio_DiscreteWrite(&Gpio, 1, currentLActivate | (newVolume << 4) | currentLNote);
	currentLVolume = newVolume;
}

//As above, but for the RIGHT audio channel
void changeRVolume(unsigned int newVolume)
{
	XGpio_DiscreteWrite(&Gpio, 2, currentRActivate | (newVolume << 4) | currentRNote);
	currentRVolume = newVolume;
}

//Change the note on the LEFT audio channel
//Do NOT change the volume or whether or not the channel is on
void changeLNote(unsigned int newNote)
{
	XGpio_DiscreteWrite(&Gpio, 1, currentLActivate | (currentLVolume << 4) | newNote);
	currentLNote = newNote;
}

//As above, but for the RIGHT audio channel
void changeRNote(unsigned int newNote)
{
	XGpio_DiscreteWrite(&Gpio, 2, currentRActivate | (currentRVolume << 4) | newNote);
	currentRNote = newNote;
}

//Turn the LEFT audio channel on or off
void Lonoff(unsigned int onoff)
{
	if(onoff)
	{
		XGpio_DiscreteWrite(&Gpio, 1, 0x80000000 | (currentLVolume << 4) | currentLNote);
		currentLActivate = 0x80000000;
	}
	else
	{
		XGpio_DiscreteWrite(&Gpio, 1, 0);
		currentLActivate = 0x00000000;
	}
}			

//Turn the RIGHT audio channel on or off
void Ronoff(unsigned int onoff)
{
	if(onoff)
	{
		XGpio_DiscreteWrite(&Gpio, 2, 0x80000000 | (currentRVolume << 4) | currentRNote);
		currentRActivate = 0x80000000;
	}
	else
	{
		XGpio_DiscreteWrite(&Gpio, 2, 0);
		currentRActivate = 0x00000000;
	}
}			

//Check the distance between the given points - if it is less than the DISTANCE_THRESHOLD, return a 0, else a 1
//This is used because if only one dot is present on the screen, the hardware will return two points very close
//to each other as the first and last points of that color present. This allows us to turn off an instrument
//if one dot is present (so by covering one of the dots used, you can turn the instrument off when playing)
int distanceCheck(int x1, int y1, int x2, int y2)
{
	double distance = sqrt( pow( (x2-x1), 2) + pow( (y2-y1), 2) );
	int dist = distance;
	if(dist > DISTANCE_THRESHOLD)
		return 1;
	else
		return 0;
}

//Takes in two points (x and y coordinates) and plays a note to the RIGHT audio channel
//based on the distance between the two points
int distance(int x1, int y1, int x2, int y2)
{
	double distance = sqrt( pow( (x2-x1), 2) + pow( (y2-y1), 2) );
	int dist = distance;

	if((dist > 45) && (dist < 83))
	{
		changeRNote(A2);//A
	}
	else if((dist > 87) && (dist < 123))
	{
		changeRNote(A2b);//B
	}
	else if((dist > 127) && (dist < 163))
	{
		changeRNote(Gb);//Db
	}
	else if((dist > 167) && (dist < 203))
	{
		changeRNote(E);//D
	}
	else if((dist > 207) && (dist < 243))
	{
		changeRNote(D);//E
	}
	else if((dist > 247) && (dist < 283))
	{
		changeRNote(Db);//Gb
	}
	else if((dist > 287) && (dist < 323))
	{
		changeRNote(B);//A2b
	}
	else if((dist > 327) && (dist < 360))
	{
		changeRNote(A);//A2
	}
}
